"""
The `operations` always inner a class and call by `@Mixin` decorator.
"""

from .accessor import Accessor as Accessor
from .accessor import FunctionAccessor as FunctionAccessor
from .hook import Both as Both
from .hook import Post as Post
from .hook import Pre as Pre
from .intercept import Intercept as Intercept
from .modify import Alias as Alias
from .modify import Del as Del
from .modify import Insert as Insert
from .modify import ReName as ReName
from .overwrite import OverWrite as OverWrite
from .processor import Processor as Processor
